#ifndef __QUEUE_H__
#define __QUEUE_H__

namespace MyQueue {
//! Excecao para indicar a condicao de underflow resultante de uma consulta/remocao em fila vazio.
class UnderflowException { /* Empty */ };

//! Excecao para indicar a condicao de overflow resultante de uma insercao em fila vazio.
class OverflowException { /* Empty */ };

template <class Object>
class Queue
{
  private:
    //! Area de armazenamento interno da fila.
    Object *mStorageArea;
    //! Indica capacidade maxima da fila.
    int     mCapacity;
    //! Indica a quantidade atual de elementos na fila.
    int     mCurrentSize;
    //! Indice do primeiro da fila.
    int     mFront;
    //! Indice do ultimo da fila.
    int     mBack;
    //! Tamanho padrao da fila, caso nada seja especificado.
    static const int DEFAULT_CAPACITY = 20;
    //! Função usada para incrementar os indices da fila, considerando uma fila circular.
    /*! @param x Indice a ser incrementado. */
    void increment( int & x ) const;

  public:
    Queue( const int capacity = DEFAULT_CAPACITY );
    Queue( const Queue& clone );
    virtual ~Queue();
    const Queue& operator= ( const Queue& rhs );

    bool isEmpty( ) const;
    const Object & front( ) const;
    const Object & back( ) const;

    void dequeue( );
    void enqueue( const Object & x );
    void clear( );
    int capacity( ) const;
    int size( ) const;
    void print( ) const;
};

}// namespace MyQueue

#include "queue.cpp"

#endif
